package com.zanxiang.sdk.common.util;

import com.zanxiang.common.exception.BaseException;

import java.util.Arrays;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * 生成签名类
 *
 * @author xufeng
 * @date 2022/7/14 17:39
 */

public class MiPayUtil {

    /**
     * 生成签名
     *
     * @param method  HTTP请求方法 "get" / "post"
     * @param urlPath CGI名字, eg: /v3/user/get_info
     * @param params  URL请求参数
     * @param secret  密钥
     * @return 签名值
     * @throws RuntimeException 不支持指定编码以及不支持指定的加密方法时抛出异常。
     */
    public static String miPaySin(String method, String urlPath, Map<String, String> params, String secret) {
        String sig;
        Object[] keys = params.keySet().toArray();
        Arrays.sort(keys);
        StringBuilder stringBuilder = new StringBuilder();
        for (int i = 0; i < keys.length; i++) {
            stringBuilder.append(keys[i]).append("=").append(params.get(keys[i]));
            if (i != keys.length - 1) {
                stringBuilder.append("&");
            }
        }
        stringBuilder.append("&org_loc=").append(urlPath)
                .append("&method=").append(method.toUpperCase())
                .append("&secret=").append(secret);
        //加密算法返回
        try {
            sig = WxPayUtil.HMACSHA256(stringBuilder.toString(), secret, Boolean.FALSE);
        } catch (Exception e) {
            throw new BaseException("米大师签名失败");
        }
        return sig;
    }


    /**
     * 应用发货URL接口对腾讯回调传来的参数value值先进行一次编码方法，用于验签
     * (编码规则为：除了 0~9 a~z A~Z !*() 之外其他字符按其ASCII码的十六进制加%进行表示，例如“-”编码为“%2D”)
     * 参考 <回调发货URL的协议说明_V3>
     *
     * @param params 腾讯回调传参Map (key,value);
     */
    public static void codePayValue(Map<String, String> params) {
        Set<String> keySet = params.keySet();
        Iterator<String> itr = keySet.iterator();

        while (itr.hasNext()) {
            String key = itr.next();
            String value = params.get(key);
            value = encodeValue(value);
            params.put(key, value);
        }
    }

    /**
     * 应用发货URL接口的编码规则
     *
     * @param s
     * @return
     */
    public static String encodeValue(String s) {
        String rexp = "[0-9a-zA-Z!*\\(\\)]";
        StringBuffer sb = new StringBuffer(s);
        StringBuffer sbRtn = new StringBuffer();
        Pattern p = Pattern.compile(rexp);
        char temp;
        String tempStr;

        for (int i = 0; i < sb.length(); i++) {
            temp = sb.charAt(i);
            tempStr = String.valueOf(temp);
            Matcher m = p.matcher(tempStr);

            boolean result = m.find();
            if (!result) {
                tempStr = hexString(tempStr);
            }
            sbRtn.append(tempStr);
        }

        return sbRtn.toString();
    }

    /**
     * 应用发货URL　十六进制编码
     *
     * @param s
     * @return
     */
    private static String hexString(String s) {
        byte[] b = s.getBytes();
        String retStr = "";
        for (int i = 0; i < b.length; i++) {
            String hex = Integer.toHexString(b[i] & 0xFF);
            if (hex.length() == 1) {
                hex = '0' + hex;
            }
            retStr = "%" + hex.toUpperCase();
        }
        return retStr;
    }

    // 编码方式
    private static final String CONTENT_CHARSET = "UTF-8";

    // HMAC算法
    private static final String HMAC_ALGORITHM = "HmacSHA256";
}
